# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network watcher test-connectivity",
    is_preview=True,
)
class TestConnectivity(AAZCommand):
    """Test if a connection can be established between a Virtual Machine and a given endpoint.

    To check connectivity between two VMs in different regions, use the VM ids instead of the VM names for the source and destination resource arguments. To register for this feature or see additional examples visit https://learn.microsoft.com/en-us/azure/network-watcher/network-watcher-connectivity-cli.

    :example: Check connectivity between two virtual machines in the same resource group over port 80.
        az network watcher test-connectivity -g MyResourceGroup --source-resource MyVmName1 --dest-resource MyVmName2 --dest-port 80

    :example: Check connectivity between two virtual machines in the same subscription in two different resource groups over port 80.
        az network watcher test-connectivity --source-resource MyVmId1 --dest-resource MyVmId2 --dest-port 80
    """

    _aaz_info = {
        "version": "2022-01-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/networkwatchers/{}/connectivitycheck", "2022-01-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.watcher_name = AAZStrArg(
            options=["--watcher-name"],
            help="Name of the network watcher resource.",
            required=True,
            id_part="name",
        )
        _args_schema.watcher_rg = AAZResourceGroupNameArg(
            options=["--watcher-rg"],
            help="Name of the resource group the watcher is in.",
            required=True,
        )
        _args_schema.protocol = AAZStrArg(
            options=["--protocol"],
            help="Protocol to test on.",
            enum={"Http": "Http", "Https": "Https", "Icmp": "Icmp", "Tcp": "Tcp"},
        )
        _args_schema.source_port = AAZIntArg(
            options=["--source-port"],
            help="Port number from which to originate traffic.",
            fmt=AAZIntArgFormat(
                maximum=65535,
                minimum=0,
            ),
        )
        _args_schema.source_resource = AAZStrArg(
            options=["--source-resource"],
            help="Name or ID of the resource from which to originate traffic. Currently only Virtual Machines are supported.",
            required=True,
        )

        # define Arg Group "Destination"

        _args_schema = cls._args_schema
        _args_schema.dest_address = AAZStrArg(
            options=["--dest-address"],
            arg_group="Destination",
            help="IP address or URI at which to receive traffic.",
        )
        _args_schema.dest_port = AAZIntArg(
            options=["--dest-port"],
            arg_group="Destination",
            help="Port number on which to receive traffic.",
            fmt=AAZIntArgFormat(
                maximum=65535,
                minimum=0,
            ),
        )
        _args_schema.dest_resource = AAZStrArg(
            options=["--dest-resource"],
            arg_group="Destination",
            help="Name or ID of the resource to receive traffic. Currently only Virtual Machines are supported.",
        )

        # define Arg Group "HTTP Configuration"

        _args_schema = cls._args_schema
        _args_schema.headers_obj = AAZListArg(
            options=["--headers-obj"],
            arg_group="HTTP Configuration",
            help="List of HTTP headers.",
        )
        _args_schema.method = AAZStrArg(
            options=["--method"],
            arg_group="HTTP Configuration",
            help="HTTP method to use.",
            enum={"Get": "Get"},
        )
        _args_schema.valid_status_codes = AAZListArg(
            options=["--valid-status-codes"],
            arg_group="HTTP Configuration",
            help="Space-separated list of HTTP status codes considered valid.",
        )

        headers_obj = cls._args_schema.headers_obj
        headers_obj.Element = AAZObjectArg()

        _element = cls._args_schema.headers_obj.Element
        _element.name = AAZStrArg(
            options=["name"],
            help="The name in HTTP header.",
        )
        _element.value = AAZStrArg(
            options=["value"],
            help="The value in HTTP header.",
        )

        valid_status_codes = cls._args_schema.valid_status_codes
        valid_status_codes.Element = AAZIntArg()

        # define Arg Group "Parameters"
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        yield self.NetworkWatchersCheckConnectivity(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class NetworkWatchersCheckConnectivity(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/networkWatchers/{networkWatcherName}/connectivityCheck",
                **self.url_parameters
            )

        @property
        def method(self):
            return "POST"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "networkWatcherName", self.ctx.args.watcher_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.watcher_rg,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("destination", AAZObjectType, ".", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("protocol", AAZStrType, ".protocol")
            _builder.set_prop("protocolConfiguration", AAZObjectType)
            _builder.set_prop("source", AAZObjectType, ".", typ_kwargs={"flags": {"required": True}})

            destination = _builder.get(".destination")
            if destination is not None:
                destination.set_prop("address", AAZStrType, ".dest_address")
                destination.set_prop("port", AAZIntType, ".dest_port")
                destination.set_prop("resourceId", AAZStrType, ".dest_resource")

            protocol_configuration = _builder.get(".protocolConfiguration")
            if protocol_configuration is not None:
                protocol_configuration.set_prop("HTTPConfiguration", AAZObjectType)

            http_configuration = _builder.get(".protocolConfiguration.HTTPConfiguration")
            if http_configuration is not None:
                http_configuration.set_prop("headers", AAZListType, ".headers_obj")
                http_configuration.set_prop("method", AAZStrType, ".method")
                http_configuration.set_prop("validStatusCodes", AAZListType, ".valid_status_codes")

            headers = _builder.get(".protocolConfiguration.HTTPConfiguration.headers")
            if headers is not None:
                headers.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".protocolConfiguration.HTTPConfiguration.headers[]")
            if _elements is not None:
                _elements.set_prop("name", AAZStrType, ".name")
                _elements.set_prop("value", AAZStrType, ".value")

            valid_status_codes = _builder.get(".protocolConfiguration.HTTPConfiguration.validStatusCodes")
            if valid_status_codes is not None:
                valid_status_codes.set_elements(AAZIntType, ".")

            source = _builder.get(".source")
            if source is not None:
                source.set_prop("port", AAZIntType, ".source_port")
                source.set_prop("resourceId", AAZStrType, ".source_resource", typ_kwargs={"flags": {"required": True}})

            return self.serialize_content(_content_value)

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _TestConnectivityHelper._build_schema_connectivity_information_read(cls._schema_on_200)

            return cls._schema_on_200


class _TestConnectivityHelper:
    """Helper class for TestConnectivity"""

    _schema_connectivity_information_read = None

    @classmethod
    def _build_schema_connectivity_information_read(cls, _schema):
        if cls._schema_connectivity_information_read is not None:
            _schema.avg_latency_in_ms = cls._schema_connectivity_information_read.avg_latency_in_ms
            _schema.connection_status = cls._schema_connectivity_information_read.connection_status
            _schema.hops = cls._schema_connectivity_information_read.hops
            _schema.max_latency_in_ms = cls._schema_connectivity_information_read.max_latency_in_ms
            _schema.min_latency_in_ms = cls._schema_connectivity_information_read.min_latency_in_ms
            _schema.probes_failed = cls._schema_connectivity_information_read.probes_failed
            _schema.probes_sent = cls._schema_connectivity_information_read.probes_sent
            return

        cls._schema_connectivity_information_read = _schema_connectivity_information_read = AAZObjectType()

        connectivity_information_read = _schema_connectivity_information_read
        connectivity_information_read.avg_latency_in_ms = AAZIntType(
            serialized_name="avgLatencyInMs",
            flags={"read_only": True},
        )
        connectivity_information_read.connection_status = AAZStrType(
            serialized_name="connectionStatus",
            flags={"read_only": True},
        )
        connectivity_information_read.hops = AAZListType(
            flags={"read_only": True},
        )
        connectivity_information_read.max_latency_in_ms = AAZIntType(
            serialized_name="maxLatencyInMs",
            flags={"read_only": True},
        )
        connectivity_information_read.min_latency_in_ms = AAZIntType(
            serialized_name="minLatencyInMs",
            flags={"read_only": True},
        )
        connectivity_information_read.probes_failed = AAZIntType(
            serialized_name="probesFailed",
            flags={"read_only": True},
        )
        connectivity_information_read.probes_sent = AAZIntType(
            serialized_name="probesSent",
            flags={"read_only": True},
        )

        hops = _schema_connectivity_information_read.hops
        hops.Element = AAZObjectType()

        _element = _schema_connectivity_information_read.hops.Element
        _element.address = AAZStrType(
            flags={"read_only": True},
        )
        _element.id = AAZStrType(
            flags={"read_only": True},
        )
        _element.issues = AAZListType(
            flags={"read_only": True},
        )
        _element.links = AAZListType(
            flags={"read_only": True},
        )
        _element.next_hop_ids = AAZListType(
            serialized_name="nextHopIds",
            flags={"read_only": True},
        )
        _element.previous_hop_ids = AAZListType(
            serialized_name="previousHopIds",
            flags={"read_only": True},
        )
        _element.previous_links = AAZListType(
            serialized_name="previousLinks",
            flags={"read_only": True},
        )
        _element.resource_id = AAZStrType(
            serialized_name="resourceId",
            flags={"read_only": True},
        )
        _element.type = AAZStrType(
            flags={"read_only": True},
        )

        issues = _schema_connectivity_information_read.hops.Element.issues
        issues.Element = AAZObjectType()
        cls._build_schema_connectivity_issue_read(issues.Element)

        links = _schema_connectivity_information_read.hops.Element.links
        links.Element = AAZObjectType()
        cls._build_schema_hop_link_read(links.Element)

        next_hop_ids = _schema_connectivity_information_read.hops.Element.next_hop_ids
        next_hop_ids.Element = AAZStrType()

        previous_hop_ids = _schema_connectivity_information_read.hops.Element.previous_hop_ids
        previous_hop_ids.Element = AAZStrType()

        previous_links = _schema_connectivity_information_read.hops.Element.previous_links
        previous_links.Element = AAZObjectType()
        cls._build_schema_hop_link_read(previous_links.Element)

        _schema.avg_latency_in_ms = cls._schema_connectivity_information_read.avg_latency_in_ms
        _schema.connection_status = cls._schema_connectivity_information_read.connection_status
        _schema.hops = cls._schema_connectivity_information_read.hops
        _schema.max_latency_in_ms = cls._schema_connectivity_information_read.max_latency_in_ms
        _schema.min_latency_in_ms = cls._schema_connectivity_information_read.min_latency_in_ms
        _schema.probes_failed = cls._schema_connectivity_information_read.probes_failed
        _schema.probes_sent = cls._schema_connectivity_information_read.probes_sent

    _schema_connectivity_issue_read = None

    @classmethod
    def _build_schema_connectivity_issue_read(cls, _schema):
        if cls._schema_connectivity_issue_read is not None:
            _schema.context = cls._schema_connectivity_issue_read.context
            _schema.origin = cls._schema_connectivity_issue_read.origin
            _schema.severity = cls._schema_connectivity_issue_read.severity
            _schema.type = cls._schema_connectivity_issue_read.type
            return

        cls._schema_connectivity_issue_read = _schema_connectivity_issue_read = AAZObjectType()

        connectivity_issue_read = _schema_connectivity_issue_read
        connectivity_issue_read.context = AAZListType(
            flags={"read_only": True},
        )
        connectivity_issue_read.origin = AAZStrType(
            flags={"read_only": True},
        )
        connectivity_issue_read.severity = AAZStrType(
            flags={"read_only": True},
        )
        connectivity_issue_read.type = AAZStrType(
            flags={"read_only": True},
        )

        context = _schema_connectivity_issue_read.context
        context.Element = AAZDictType()

        _element = _schema_connectivity_issue_read.context.Element
        _element.Element = AAZStrType()

        _schema.context = cls._schema_connectivity_issue_read.context
        _schema.origin = cls._schema_connectivity_issue_read.origin
        _schema.severity = cls._schema_connectivity_issue_read.severity
        _schema.type = cls._schema_connectivity_issue_read.type

    _schema_hop_link_read = None

    @classmethod
    def _build_schema_hop_link_read(cls, _schema):
        if cls._schema_hop_link_read is not None:
            _schema.context = cls._schema_hop_link_read.context
            _schema.issues = cls._schema_hop_link_read.issues
            _schema.link_type = cls._schema_hop_link_read.link_type
            _schema.next_hop_id = cls._schema_hop_link_read.next_hop_id
            _schema.properties = cls._schema_hop_link_read.properties
            _schema.resource_id = cls._schema_hop_link_read.resource_id
            return

        cls._schema_hop_link_read = _schema_hop_link_read = AAZObjectType()

        hop_link_read = _schema_hop_link_read
        hop_link_read.context = AAZDictType(
            flags={"read_only": True},
        )
        hop_link_read.issues = AAZListType(
            flags={"read_only": True},
        )
        hop_link_read.link_type = AAZStrType(
            serialized_name="linkType",
            flags={"read_only": True},
        )
        hop_link_read.next_hop_id = AAZStrType(
            serialized_name="nextHopId",
            flags={"read_only": True},
        )
        hop_link_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        hop_link_read.resource_id = AAZStrType(
            serialized_name="resourceId",
            flags={"read_only": True},
        )

        context = _schema_hop_link_read.context
        context.Element = AAZStrType()

        issues = _schema_hop_link_read.issues
        issues.Element = AAZObjectType()
        cls._build_schema_connectivity_issue_read(issues.Element)

        properties = _schema_hop_link_read.properties
        properties.round_trip_time_avg = AAZIntType(
            serialized_name="roundTripTimeAvg",
            flags={"read_only": True},
        )
        properties.round_trip_time_max = AAZIntType(
            serialized_name="roundTripTimeMax",
            flags={"read_only": True},
        )
        properties.round_trip_time_min = AAZIntType(
            serialized_name="roundTripTimeMin",
            flags={"read_only": True},
        )

        _schema.context = cls._schema_hop_link_read.context
        _schema.issues = cls._schema_hop_link_read.issues
        _schema.link_type = cls._schema_hop_link_read.link_type
        _schema.next_hop_id = cls._schema_hop_link_read.next_hop_id
        _schema.properties = cls._schema_hop_link_read.properties
        _schema.resource_id = cls._schema_hop_link_read.resource_id


__all__ = ["TestConnectivity"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "image create",
)
class Create(AAZCommand):
    """Create a custom Virtual Machine Image from managed disks or snapshots.

    :example: Create an image from an existing disk.
        az image create -g MyResourceGroup -n image1 --os-type Linux --source /subscriptions/db5eb68e-73e2-4fa8-b18a-0123456789999/resourceGroups/rg1/providers/Microsoft.Compute/snapshots/s1

    :example: Create an image by capturing an existing generalized virtual machine in the same resource group.
        az image create -g MyResourceGroup -n image1 --source MyVm1
    """

    _aaz_info = {
        "version": "2024-07-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.compute/images/{}", "2024-07-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.image_name = AAZStrArg(
            options=["-n", "--name", "--image-name"],
            help="The name of the image.",
            required=True,
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "Parameters"

        _args_schema = cls._args_schema
        _args_schema.extended_location = AAZObjectArg(
            options=["--extended-location"],
            arg_group="Parameters",
            help="The extended location of the Image.",
        )
        _args_schema.location = AAZResourceLocationArg(
            arg_group="Parameters",
            help="Resource location",
            required=True,
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            arg_group="Parameters",
            help="Resource tags",
        )

        extended_location = cls._args_schema.extended_location
        extended_location.name = AAZStrArg(
            options=["name"],
            help="The name of the extended location.",
        )
        extended_location.type = AAZStrArg(
            options=["type"],
            help="The type of the extended location.",
            enum={"EdgeZone": "EdgeZone"},
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg()

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.hyper_v_generation = AAZStrArg(
            options=["--hyper-v-generation"],
            arg_group="Properties",
            help="The hypervisor generation of the Virtual Machine created from the image.",
            default="V1",
            enum={"V1": "V1", "V2": "V2"},
        )
        _args_schema.source_virtual_machine = AAZObjectArg(
            options=["--source-virtual-machine"],
            arg_group="Properties",
            help="The source virtual machine from which Image is created.",
        )
        _args_schema.storage_profile = AAZObjectArg(
            options=["--storage-profile"],
            arg_group="Properties",
            help="Specifies the storage settings for the virtual machine disks.",
        )

        source_virtual_machine = cls._args_schema.source_virtual_machine
        source_virtual_machine.id = AAZStrArg(
            options=["id"],
            help="Resource Id",
        )

        storage_profile = cls._args_schema.storage_profile
        storage_profile.data_disks = AAZListArg(
            options=["data-disks"],
            help="Specifies the parameters that are used to add a data disk to a virtual machine. <br><br> For more information about disks, see [About disks and VHDs for Azure virtual machines](https://learn.microsoft.com/azure/virtual-machines/managed-disks-overview).",
        )
        storage_profile.os_disk = AAZObjectArg(
            options=["os-disk"],
            help="Specifies information about the operating system disk used by the virtual machine. <br><br> For more information about disks, see [About disks and VHDs for Azure virtual machines](https://learn.microsoft.com/azure/virtual-machines/managed-disks-overview).",
        )
        storage_profile.zone_resilient = AAZBoolArg(
            options=["zone-resilient"],
            help="Specifies whether an image is zone resilient or not. Default is false. Zone resilient images can be created only in regions that provide Zone Redundant Storage (ZRS).",
        )

        data_disks = cls._args_schema.storage_profile.data_disks
        data_disks.Element = AAZObjectArg()

        _element = cls._args_schema.storage_profile.data_disks.Element
        _element.blob_uri = AAZStrArg(
            options=["blob-uri"],
            help="The Virtual Hard Disk.",
        )
        _element.caching = AAZStrArg(
            options=["caching"],
            help="Specifies the caching requirements. <br><br> Possible values are: <br><br> **None** <br><br> **ReadOnly** <br><br> **ReadWrite** <br><br> Default: **None for Standard storage. ReadOnly for Premium storage**",
            enum={"None": "None", "ReadOnly": "ReadOnly", "ReadWrite": "ReadWrite"},
        )
        _element.disk_encryption_set = AAZObjectArg(
            options=["disk-encryption-set"],
            help="Specifies the customer managed disk encryption set resource id for the managed image disk.",
        )
        cls._build_args_disk_encryption_set_parameters_create(_element.disk_encryption_set)
        _element.disk_size_gb = AAZIntArg(
            options=["disk-size-gb"],
            help="Specifies the size of empty data disks in gigabytes. This element can be used to overwrite the name of the disk in a virtual machine image. <br><br> This value cannot be larger than 1023 GB",
        )
        _element.lun = AAZIntArg(
            options=["lun"],
            help="Specifies the logical unit number of the data disk. This value is used to identify data disks within the VM and therefore must be unique for each data disk attached to a VM.",
            required=True,
        )
        _element.managed_disk = AAZObjectArg(
            options=["managed-disk"],
            help="The managedDisk.",
        )
        _element.snapshot = AAZObjectArg(
            options=["snapshot"],
            help="The snapshot.",
        )
        _element.storage_account_type = AAZStrArg(
            options=["storage-account-type"],
            help="Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS can only be used with data disks, it cannot be used with OS Disk.",
            enum={"PremiumV2_LRS": "PremiumV2_LRS", "Premium_LRS": "Premium_LRS", "Premium_ZRS": "Premium_ZRS", "StandardSSD_LRS": "StandardSSD_LRS", "StandardSSD_ZRS": "StandardSSD_ZRS", "Standard_LRS": "Standard_LRS", "UltraSSD_LRS": "UltraSSD_LRS"},
        )

        managed_disk = cls._args_schema.storage_profile.data_disks.Element.managed_disk
        managed_disk.id = AAZStrArg(
            options=["id"],
        )

        snapshot = cls._args_schema.storage_profile.data_disks.Element.snapshot
        snapshot.id = AAZStrArg(
            options=["id"],
        )

        os_disk = cls._args_schema.storage_profile.os_disk
        os_disk.blob_uri = AAZStrArg(
            options=["blob-uri"],
            help="The Virtual Hard Disk.",
        )
        os_disk.caching = AAZStrArg(
            options=["caching"],
            help="Specifies the caching requirements. <br><br> Possible values are: <br><br> **None** <br><br> **ReadOnly** <br><br> **ReadWrite** <br><br> Default: **None for Standard storage. ReadOnly for Premium storage**",
            enum={"None": "None", "ReadOnly": "ReadOnly", "ReadWrite": "ReadWrite"},
        )
        os_disk.disk_encryption_set = AAZObjectArg(
            options=["disk-encryption-set"],
            help="Specifies the customer managed disk encryption set resource id for the managed image disk.",
        )
        cls._build_args_disk_encryption_set_parameters_create(os_disk.disk_encryption_set)
        os_disk.disk_size_gb = AAZIntArg(
            options=["disk-size-gb"],
            help="Specifies the size of empty data disks in gigabytes. This element can be used to overwrite the name of the disk in a virtual machine image. <br><br> This value cannot be larger than 1023 GB",
        )
        os_disk.managed_disk = AAZObjectArg(
            options=["managed-disk"],
            help="The managedDisk.",
        )
        cls._build_args_sub_resource_create(os_disk.managed_disk)
        os_disk.os_state = AAZStrArg(
            options=["os-state"],
            help="The OS State. For managed images, use Generalized.",
            required=True,
            enum={"Generalized": "Generalized", "Specialized": "Specialized"},
        )
        os_disk.os_type = AAZStrArg(
            options=["os-type"],
            help="This property allows you to specify the type of the OS that is included in the disk if creating a VM from a custom image. <br><br> Possible values are: <br><br> **Windows** <br><br> **Linux**",
            required=True,
            enum={"Linux": "Linux", "Windows": "Windows"},
        )
        os_disk.snapshot = AAZObjectArg(
            options=["snapshot"],
            help="The snapshot.",
        )
        os_disk.storage_account_type = AAZStrArg(
            options=["storage-account-type"],
            help="Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS can only be used with data disks, it cannot be used with OS Disk.",
            enum={"PremiumV2_LRS": "PremiumV2_LRS", "Premium_LRS": "Premium_LRS", "Premium_ZRS": "Premium_ZRS", "StandardSSD_LRS": "StandardSSD_LRS", "StandardSSD_ZRS": "StandardSSD_ZRS", "Standard_LRS": "Standard_LRS", "UltraSSD_LRS": "UltraSSD_LRS"},
        )

        snapshot = cls._args_schema.storage_profile.os_disk.snapshot
        snapshot.id = AAZStrArg(
            options=["id"],
        )
        return cls._args_schema

    _args_disk_encryption_set_parameters_create = None

    @classmethod
    def _build_args_disk_encryption_set_parameters_create(cls, _schema):
        if cls._args_disk_encryption_set_parameters_create is not None:
            _schema.id = cls._args_disk_encryption_set_parameters_create.id
            return

        cls._args_disk_encryption_set_parameters_create = AAZObjectArg()

        disk_encryption_set_parameters_create = cls._args_disk_encryption_set_parameters_create
        disk_encryption_set_parameters_create.id = AAZStrArg(
            options=["id"],
            help="Resource Id",
        )

        _schema.id = cls._args_disk_encryption_set_parameters_create.id

    _args_sub_resource_create = None

    @classmethod
    def _build_args_sub_resource_create(cls, _schema):
        if cls._args_sub_resource_create is not None:
            _schema.id = cls._args_sub_resource_create.id
            return

        cls._args_sub_resource_create = AAZObjectArg()

        sub_resource_create = cls._args_sub_resource_create
        sub_resource_create.id = AAZStrArg(
            options=["id"],
            help="Resource Id",
        )

        _schema.id = cls._args_sub_resource_create.id

    def _execute_operations(self):
        self.pre_operations()
        yield self.ImagesCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class ImagesCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/images/{imageName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "imageName", self.ctx.args.image_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-07-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("extendedLocation", AAZObjectType, ".extended_location")
            _builder.set_prop("location", AAZStrType, ".location", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            extended_location = _builder.get(".extendedLocation")
            if extended_location is not None:
                extended_location.set_prop("name", AAZStrType, ".name")
                extended_location.set_prop("type", AAZStrType, ".type")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("hyperVGeneration", AAZStrType, ".hyper_v_generation")
                properties.set_prop("sourceVirtualMachine", AAZObjectType, ".source_virtual_machine")
                properties.set_prop("storageProfile", AAZObjectType, ".storage_profile")

            source_virtual_machine = _builder.get(".properties.sourceVirtualMachine")
            if source_virtual_machine is not None:
                source_virtual_machine.set_prop("id", AAZStrType, ".id")

            storage_profile = _builder.get(".properties.storageProfile")
            if storage_profile is not None:
                storage_profile.set_prop("dataDisks", AAZListType, ".data_disks")
                storage_profile.set_prop("osDisk", AAZObjectType, ".os_disk")
                storage_profile.set_prop("zoneResilient", AAZBoolType, ".zone_resilient")

            data_disks = _builder.get(".properties.storageProfile.dataDisks")
            if data_disks is not None:
                data_disks.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.storageProfile.dataDisks[]")
            if _elements is not None:
                _elements.set_prop("blobUri", AAZStrType, ".blob_uri")
                _elements.set_prop("caching", AAZStrType, ".caching")
                _CreateHelper._build_schema_disk_encryption_set_parameters_create(_elements.set_prop("diskEncryptionSet", AAZObjectType, ".disk_encryption_set"))
                _elements.set_prop("diskSizeGB", AAZIntType, ".disk_size_gb")
                _elements.set_prop("lun", AAZIntType, ".lun", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("managedDisk", AAZObjectType, ".managed_disk")
                _elements.set_prop("snapshot", AAZObjectType, ".snapshot")
                _elements.set_prop("storageAccountType", AAZStrType, ".storage_account_type")

            managed_disk = _builder.get(".properties.storageProfile.dataDisks[].managedDisk")
            if managed_disk is not None:
                managed_disk.set_prop("id", AAZStrType, ".id")

            snapshot = _builder.get(".properties.storageProfile.dataDisks[].snapshot")
            if snapshot is not None:
                snapshot.set_prop("id", AAZStrType, ".id")

            os_disk = _builder.get(".properties.storageProfile.osDisk")
            if os_disk is not None:
                os_disk.set_prop("blobUri", AAZStrType, ".blob_uri")
                os_disk.set_prop("caching", AAZStrType, ".caching")
                _CreateHelper._build_schema_disk_encryption_set_parameters_create(os_disk.set_prop("diskEncryptionSet", AAZObjectType, ".disk_encryption_set"))
                os_disk.set_prop("diskSizeGB", AAZIntType, ".disk_size_gb")
                _CreateHelper._build_schema_sub_resource_create(os_disk.set_prop("managedDisk", AAZObjectType, ".managed_disk"))
                os_disk.set_prop("osState", AAZStrType, ".os_state", typ_kwargs={"flags": {"required": True}})
                os_disk.set_prop("osType", AAZStrType, ".os_type", typ_kwargs={"flags": {"required": True}})
                os_disk.set_prop("snapshot", AAZObjectType, ".snapshot")
                os_disk.set_prop("storageAccountType", AAZStrType, ".storage_account_type")

            snapshot = _builder.get(".properties.storageProfile.osDisk.snapshot")
            if snapshot is not None:
                snapshot.set_prop("id", AAZStrType, ".id")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()

            _schema_on_200_201 = cls._schema_on_200_201
            _schema_on_200_201.extended_location = AAZObjectType(
                serialized_name="extendedLocation",
            )
            _schema_on_200_201.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.location = AAZStrType(
                flags={"required": True},
            )
            _schema_on_200_201.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200_201.tags = AAZDictType()
            _schema_on_200_201.type = AAZStrType(
                flags={"read_only": True},
            )

            extended_location = cls._schema_on_200_201.extended_location
            extended_location.name = AAZStrType()
            extended_location.type = AAZStrType()

            properties = cls._schema_on_200_201.properties
            properties.hyper_v_generation = AAZStrType(
                serialized_name="hyperVGeneration",
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.source_virtual_machine = AAZObjectType(
                serialized_name="sourceVirtualMachine",
            )
            _CreateHelper._build_schema_sub_resource_read(properties.source_virtual_machine)
            properties.storage_profile = AAZObjectType(
                serialized_name="storageProfile",
            )

            storage_profile = cls._schema_on_200_201.properties.storage_profile
            storage_profile.data_disks = AAZListType(
                serialized_name="dataDisks",
            )
            storage_profile.os_disk = AAZObjectType(
                serialized_name="osDisk",
            )
            storage_profile.zone_resilient = AAZBoolType(
                serialized_name="zoneResilient",
            )

            data_disks = cls._schema_on_200_201.properties.storage_profile.data_disks
            data_disks.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.storage_profile.data_disks.Element
            _element.blob_uri = AAZStrType(
                serialized_name="blobUri",
            )
            _element.caching = AAZStrType()
            _element.disk_encryption_set = AAZObjectType(
                serialized_name="diskEncryptionSet",
            )
            _CreateHelper._build_schema_disk_encryption_set_parameters_read(_element.disk_encryption_set)
            _element.disk_size_gb = AAZIntType(
                serialized_name="diskSizeGB",
            )
            _element.lun = AAZIntType(
                flags={"required": True},
            )
            _element.managed_disk = AAZObjectType(
                serialized_name="managedDisk",
            )
            _CreateHelper._build_schema_sub_resource_read(_element.managed_disk)
            _element.snapshot = AAZObjectType()
            _CreateHelper._build_schema_sub_resource_read(_element.snapshot)
            _element.storage_account_type = AAZStrType(
                serialized_name="storageAccountType",
            )

            os_disk = cls._schema_on_200_201.properties.storage_profile.os_disk
            os_disk.blob_uri = AAZStrType(
                serialized_name="blobUri",
            )
            os_disk.caching = AAZStrType()
            os_disk.disk_encryption_set = AAZObjectType(
                serialized_name="diskEncryptionSet",
            )
            _CreateHelper._build_schema_disk_encryption_set_parameters_read(os_disk.disk_encryption_set)
            os_disk.disk_size_gb = AAZIntType(
                serialized_name="diskSizeGB",
            )
            os_disk.managed_disk = AAZObjectType(
                serialized_name="managedDisk",
            )
            _CreateHelper._build_schema_sub_resource_read(os_disk.managed_disk)
            os_disk.os_state = AAZStrType(
                serialized_name="osState",
                flags={"required": True},
            )
            os_disk.os_type = AAZStrType(
                serialized_name="osType",
                flags={"required": True},
            )
            os_disk.snapshot = AAZObjectType()
            _CreateHelper._build_schema_sub_resource_read(os_disk.snapshot)
            os_disk.storage_account_type = AAZStrType(
                serialized_name="storageAccountType",
            )

            tags = cls._schema_on_200_201.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200_201


class _CreateHelper:
    """Helper class for Create"""

    @classmethod
    def _build_schema_disk_encryption_set_parameters_create(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("id", AAZStrType, ".id")

    @classmethod
    def _build_schema_sub_resource_create(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("id", AAZStrType, ".id")

    _schema_disk_encryption_set_parameters_read = None

    @classmethod
    def _build_schema_disk_encryption_set_parameters_read(cls, _schema):
        if cls._schema_disk_encryption_set_parameters_read is not None:
            _schema.id = cls._schema_disk_encryption_set_parameters_read.id
            return

        cls._schema_disk_encryption_set_parameters_read = _schema_disk_encryption_set_parameters_read = AAZObjectType()

        disk_encryption_set_parameters_read = _schema_disk_encryption_set_parameters_read
        disk_encryption_set_parameters_read.id = AAZStrType()

        _schema.id = cls._schema_disk_encryption_set_parameters_read.id

    _schema_sub_resource_read = None

    @classmethod
    def _build_schema_sub_resource_read(cls, _schema):
        if cls._schema_sub_resource_read is not None:
            _schema.id = cls._schema_sub_resource_read.id
            return

        cls._schema_sub_resource_read = _schema_sub_resource_read = AAZObjectType()

        sub_resource_read = _schema_sub_resource_read
        sub_resource_read.id = AAZStrType()

        _schema.id = cls._schema_sub_resource_read.id


__all__ = ["Create"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from collections.abc import Iterable
from azure.cli.command_modules.keyvault._validators import KeyEncryptionDataType


def multi_transformers(*transformers):
    def _multi_transformers(output, **command_args):
        for t in transformers:
            output = t(output, **command_args)
        return output
    return _multi_transformers


def keep_max_results(output, **command_args):
    maxresults = command_args.get('maxresults', command_args.get('max_page_size', None))
    if maxresults:
        return list(output)[:maxresults]
    return output


def filter_out_managed_resources(output, **command_args):  # pylint: disable=unused-argument
    is_kv_transform = command_args.get('kv_transform')
    if is_kv_transform:
        if command_args.get('include_managed'):
            return output
        return [_ for _ in output if not getattr(_, 'managed')] if output else output
    return output


def _extract_subresource_name_from_single_output(output, id_parameter):
    if not getattr(output, id_parameter):
        resource_name = None
    else:
        items = getattr(output, id_parameter).split('/')
        resource_name = items[4] if len(items) > 4 else None

    setattr(output, 'name', resource_name)
    return output


def extract_subresource_name(id_parameter='id'):
    def _extract_subresource_name(output, **command_args):  # pylint: disable=unused-argument
        if isinstance(output, Iterable):
            return [_extract_subresource_name_from_single_output(item, id_parameter) for item in output]
        return _extract_subresource_name_from_single_output(output, id_parameter)
    return _extract_subresource_name


def transform_key_encryption_output(result, **command_args):  # pylint: disable=unused-argument
    if not result or isinstance(result, dict):
        return result

    # transform EncryptResult to dict
    import base64
    import binascii
    output = {
        'kid': result.key_id,
        'result': base64.b64encode(result.ciphertext).decode('utf-8'),
        'algorithm': result.algorithm,
        'iv': binascii.hexlify(result.iv) if result.iv else None,
        'tag': binascii.hexlify(result.tag) if result.tag else None,
        'aad': binascii.hexlify(result.aad) if result.aad else None
    }
    return output


def transform_key_decryption_output(result, **command_args):
    if not result or isinstance(result, dict):
        return result

    # transform DecryptResult to dict
    import base64
    output = {
        'kid': result.key_id,
        'result': result.plaintext,
        'algorithm': result.algorithm
    }
    data_type = command_args.get('data_type')
    if data_type == KeyEncryptionDataType.BASE64:
        output['result'] = base64.b64encode(result.plaintext).decode('utf-8')
    return output


def transform_key_list_output(result, **command_args):  # pylint: disable=unused-argument
    if not result:
        return result
    output = []
    from azure.keyvault.keys import KeyProperties, DeletedKey
    for key in result:
        if not isinstance(key, KeyProperties) and not isinstance(key, DeletedKey):
            return result
        k = {}
        if isinstance(key, DeletedKey):
            k['deletedDate'] = key.deleted_date
            k['scheduledPurgeDate'] = key.scheduled_purge_date
            k['recoveryId'] = key.recovery_id
            key = key.properties
        k['attributes'] = key._attributes    # pylint: disable=protected-access
        k['kid'] = key.id
        k['name'] = key.name
        k['managed'] = key.managed
        k['tags'] = key.tags
        k['releasePolicy'] = key.release_policy
        output.append(k)
    return output


# pylint: disable=unused-argument, protected-access
def transform_key_output(result, **command_args):
    from azure.keyvault.keys import KeyVaultKey, DeletedKey, JsonWebKey
    import base64

    if not isinstance(result, KeyVaultKey) and not isinstance(result, DeletedKey):
        return result

    if result.key and isinstance(result.key, JsonWebKey):
        for attr in result.key._FIELDS:
            value = getattr(result.key, attr)
            if value and isinstance(value, bytes):
                setattr(result.key, attr, base64.b64encode(value))

    output = {
        'attributes': {
            'created': result.properties.created_on,
            'enabled': result.properties.enabled,
            'expires': result.properties.expires_on,
            'exportable': result.properties.exportable,
            'hsmPlatform': result.properties.hsm_platform,
            'notBefore': result.properties.not_before,
            'recoverableDays': result.properties.recoverable_days,
            'recoveryLevel': result.properties.recovery_level,
            'updated': result.properties.updated_on
        },
        'key': result.key,
        'managed': result.properties.managed,
        'tags': result.properties.tags,
        'releasePolicy': result.properties.release_policy
    }
    if isinstance(result, DeletedKey):
        output['deletedDate'] = result.deleted_date
        output['scheduledPurgeDate'] = result.scheduled_purge_date
        output['recoveryId'] = result.recovery_id
    return output


# pylint: disable=unused-argument
def transform_key_random_output(result, **command_args):
    if result and isinstance(result, bytes):
        import base64
        return {"value": base64.b64encode(result)}

    return result


def transform_secret_list(result, **command_args):
    return [transform_secret_base_properties(secret) for secret in result]


def transform_secret_base_properties(result, **command_args):
    if not isinstance(result, dict):
        ret = {
            "attributes": getattr(result, "_attributes", None),
            "contentType": getattr(result, "content_type", None),
            "id": getattr(result, "id", None),
            "managed": getattr(result, "managed", None),
            "name": getattr(result, "name", None),
            "tags": getattr(result, "tags", None)
        }
        return ret
    return result


def transform_deleted_secret_list(result, **command_args):
    return [transform_deleted_secret_properties(secret) for secret in result]


def transform_deleted_secret_properties(result):
    if not isinstance(result, dict):
        ret = transform_secret_base_properties(getattr(result, "properties", None))
        ret.update({
            "deletedDate": getattr(result, "deleted_date", None),
            "recoveryId": getattr(result, "recovery_id", None),
            "scheduledPurgeDate": getattr(result, "scheduled_purge_date", None)
        })
        return ret
    return result


def transform_secret_set(result, **command_args):
    if not isinstance(result, dict):
        properties = getattr(result, "properties", None)
        ret = transform_secret_base_properties(properties)
        ret.update({
            "kid": getattr(properties, "key_id", None),
            "value": getattr(result, "value", None)
        })
        return ret
    return result


def transform_secret_set_attributes(result, **command_args):
    if not isinstance(result, dict):
        ret = transform_secret_base_properties(result)
        ret.update({
            "kid": getattr(result, "key_id", None),
            "value": None
        })
        return ret
    return result


def transform_secret_show_deleted(result, **command_args):
    if not isinstance(result, dict):
        properties = getattr(result, "properties", None)
        ret = transform_secret_base_properties(properties)
        ret.update({
            "kid": getattr(properties, "key_id", None),
            "value": None,
            "deletedDate": getattr(result, "deleted_date", None),
            "recoveryId": getattr(result, "recovery_id", None),
            "scheduledPurgeDate": getattr(result, "scheduled_purge_date", None)
        })
        return ret
    return result


def transform_secret_delete(result, **command_args):
    if not isinstance(result, dict):
        ret = transform_secret_show_deleted(result.result())
        return ret
    return result


def transform_secret_recover(result, **command_args):
    if not isinstance(result, dict):
        ret = transform_secret_set_attributes(result.result())
        return ret
    return result


def transform_certificate_create(result, **command_args):
    if not isinstance(result, dict):
        import base64
        csr = getattr(result, "csr", None)
        ret = {
            "cancellationRequested": getattr(result, "cancellation_requested", None),
            "csr": base64.b64encode(csr).decode('utf-8') if csr else None,
            "error": getattr(result, "error", None),
            "id": getattr(result, "id", None),
            "issuerParameters": {
                "certificateTransparency": getattr(result, "certificate_transparency", None),
                "certificateType": getattr(result, "certificate_type", None),
                "name": getattr(result, "issuer_name", None)
            },
            "name": getattr(result, "name", None),
            "requestId": getattr(result, "request_id", None),
            "status": getattr(result, "status", None),
            "statusDetails": getattr(result, "status_details", None),
            "target": getattr(result, "target", None)
        }
        return ret
    return result


def transform_certificate_list(result, **command_args):
    return [transform_certificate_properties(certificate) for certificate in result]


def transform_certificate_properties(result):
    import base64
    if not isinstance(result, dict):
        x509_thumbprint = getattr(result, "x509_thumbprint", None)
        ret = {
            "attributes": {
                "created": getattr(result, "created_on", None),
                "enabled": getattr(result, "enabled", None),
                "expires": getattr(result, "expires_on", None),
                "notBefore": getattr(result, "not_before", None),
                "recoveryLevel": getattr(result, "recovery_level", None),
                "updated": getattr(result, "updated_on", None)
            },
            "id": getattr(result, "id", None),
            "name": getattr(result, "name", None),
            "subject": getattr(result, "subject", ""),
            "tags": getattr(result, "tags", None),
            "x509Thumbprint": base64.b64encode(x509_thumbprint).decode('utf-8') if x509_thumbprint else None,
            "x509ThumbprintHex": x509_thumbprint.hex().upper() if x509_thumbprint else None
        }
        return ret
    return result


def transform_certificate_list_deleted(result, **command_args):
    return [transform_deleted_certificate(certificate) for certificate in result]


def transform_deleted_certificate(result):
    if not isinstance(result, dict):
        ret = transform_certificate_properties(getattr(result, "properties", None))
        ret.update({
            "deletedDate": getattr(result, "deleted_on", None),
            "recoveryId": getattr(result, "recovery_id", None),
            "scheduledPurgeDate": getattr(result, "scheduled_purge_date", None)
        })
        del ret["subject"]
        return ret
    return result


def transform_certificate_show(result, **command_args):
    import base64
    if not isinstance(result, dict):
        ret = transform_certificate_properties(getattr(result, "properties", None))
        cert_id = getattr(result, "id", None)
        cer = getattr(result, "cer", None)
        ret.update({
            "cer": base64.b64encode(cer).decode('utf-8') if cer else None,
            "contentType": getattr(result, "content_type", None),
            "kid": getattr(result, "key_id", None),
            "policy": transform_certificate_policy(policy=getattr(result, "policy", None),
                                                   policy_id='/'.join(cert_id.split('/')[:-1] + ['policy'])),
            "sid": getattr(result, "secret_id", None)
        })
        del ret["subject"]
        return ret
    return result


# pylint: disable=line-too-long,redefined-builtin
def transform_certificate_policy(policy, policy_id):
    if policy is not None and not isinstance(policy, dict):
        policy = {
            "attributes": {
                "created": getattr(policy, "created_on", None),
                "enabled": getattr(policy, "enabled", None),
                "expires": getattr(policy, "expires_on", None),
                "notBefore": getattr(policy, "not_before", None),
                "recoveryLevel": getattr(policy, "recovery_level", None),
                "updated": getattr(policy, "updated_on", None)
            },
            "id": policy_id,
            "issuerParameters": {
                "certificateTransparency": getattr(policy, "certificate_transparency", None),
                "certificateType": getattr(policy, "certificate_type", None),
                "name": getattr(policy, "issuer_name", None)
            },
            "keyProperties": {
                "curve": getattr(policy, "curve", None),
                "exportable": getattr(policy, "exportable", None),
                "keySize": getattr(policy, "key_size", None),
                "keyType": getattr(policy, "key_type", None),
                "reuseKey": getattr(policy, "reuse_key", None)
            },
            "lifetimeActions": [{
                "action": {
                    "actionType": getattr(action, "action", None)
                },
                "trigger": {
                    "daysBeforeExpiry": getattr(action, "days_before_expiry", None),
                    "lifetimePercentage": getattr(action, "lifetime_percentage", None)
                }
            } for action in (getattr(policy, "lifetime_actions", None) or [])],
            "secretProperties": {
                "contentType": getattr(policy, "content_type", None)
            },
            "x509CertificateProperties": {
                "ekus": getattr(policy, "enhanced_key_usage", None),
                "keyUsage": getattr(policy, "key_usage", None),
                "subject": getattr(policy, "subject", None),
                "subjectAlternativeNames": {
                    "emails": getattr(policy, "san_emails", None),
                    "upns": getattr(policy, "san_user_principal_names", None),
                    "dnsNames": getattr(policy, "san_dns_names", None)
                },
                "validityInMonths": getattr(policy, "validity_in_months", None)
            }
        }
        return policy
    return policy


def transform_certificate_show_deleted(result, **command_args):
    if not isinstance(result, dict):
        ret = transform_certificate_show(result)
        ret.update({
            "deletedDate": getattr(result, "deleted_on", None),
            "recoveryId": getattr(result, "recovery_id", None),
            "scheduledPurgeDate": getattr(result, "scheduled_purge_date", None)
        })
        return ret
    return result


def transform_certificate_delete(result, **command_args):
    if not isinstance(result, dict):
        ret = transform_certificate_show_deleted(result.result())
        return ret
    return result


def transform_certificate_recover(result, **command_args):
    if not isinstance(result, dict):
        ret = transform_certificate_show(result.result())
        return ret
    return result


def transform_certificate_contact_list(result, **command_args):
    if isinstance(result, list):
        client = command_args.get('self')
        return transform_certificate_contact_list_result(result, client)
    return result


def transform_certificate_contact_list_result(result, client):
    contacts_id = ""
    if getattr(client, "vault_url") and isinstance(getattr(client, "vault_url"), str):
        contacts_id = getattr(client, "vault_url") + '/certificates/contacts'
    ret = {
        "contactList": [
            {
                "emailAddress": getattr(contact, "email"),
                "name": getattr(contact, "name"),
                "phone": getattr(contact, "phone")
            } for contact in result
        ],
        "id": contacts_id
    }
    return ret


def transform_certificate_contact_add(result, **command_args):
    if isinstance(result, list):
        client = command_args.get('client')
        return transform_certificate_contact_list_result(result, client)
    return result


def transform_certificate_issuer_create(result, **command_args):
    if not isinstance(result, dict):
        ret = {
            "attributes": {
                "created": getattr(result, "created_on"),
                "enabled": getattr(result, "enabled"),
                "updated": getattr(result, "updated_on")
            },
            "credentials": {
                "accountId": getattr(result, "account_id"),
                "password": getattr(result, "password")
            },
            "id": getattr(result, "id"),
            "organizationDetails": {
                "adminDetails": getattr(result, "admin_contacts"),
                "id": getattr(result, "organization_id"),
                "zip": 0
            },
            "provider": getattr(result, "provider")
        }
        return ret
    return result


def transform_certificate_issuer_list(result, **command_args):
    if not isinstance(result, dict) and not isinstance(result, list):
        ret = [{
            "id": getattr(res, "id"),
            "provider": getattr(res, "provider")
        } for res in result]
        return ret
    return result


def transform_certificate_issuer_admin_list(result, **command_args):
    if not isinstance(result, dict) and not isinstance(result, list):
        admin_contacts = getattr(result, 'admin_contacts')
        ret = [{
            "emailAddress": getattr(contact, "email"),
            "firstName": getattr(contact, "first_name"),
            "lastName": getattr(contact, "last_name"),
            "phone": getattr(contact, "phone")
        } for contact in admin_contacts]
        return ret
    return result



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.core.breaking_change import register_default_value_breaking_change

register_default_value_breaking_change(command_name='vm create',
                                       arg='--size',
                                       current_default='Standard_DS1_v2',
                                       new_default='Standard_D2s_v5',
                                       target_version=None)

register_default_value_breaking_change(command_name='vmss create',
                                       arg='--vm-sku',
                                       current_default='Standard_DS1_v2',
                                       new_default='Standard_D2s_v5',
                                       target_version=None)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=too-many-locals, line-too-long, protected-access, too-many-nested-blocks
import antlr4

from azure.cli.command_modules.monitor.actions import AAZCustomListArg
from azure.cli.command_modules.monitor.grammar.metric_alert import MetricAlertConditionLexer, \
    MetricAlertConditionParser, MetricAlertConditionValidator
from azure.cli.core.aaz import has_value
from azure.cli.core.azclierror import InvalidArgumentValueError
from azure.mgmt.core.tools import is_valid_resource_id, resource_id
from knack.log import get_logger
from msrest.serialization import Serializer

from ..aaz.latest.monitor.metrics.alert import Update as _MetricsAlertUpdate

logger = get_logger(__name__)

_metric_alert_dimension_prefix = '_where_'


def create_metric_alert(cmd, resource_group_name, rule_name, scopes, condition, disabled=False, description=None,
                        tags=None, actions=None, severity=2, window_size='5m', evaluation_frequency='1m',
                        auto_mitigate=None, target_resource_type=None, target_resource_region=None):
    # generate metadata for the conditions
    is_dynamic_threshold_criterion = False
    all_of = []
    single_all_of = []
    for i, cond in enumerate(condition):
        if "dynamic" in cond:
            is_dynamic_threshold_criterion = True
            item = cond["dynamic"]
            item["name"] = f"cond{i}"
            props = {
                "alert_sensitivity": item.pop("alert_sensitivity", None),
                "failing_periods": item.pop("failing_periods", None),
                "operator": item.pop("operator", None),
                "ignore_data_before": Serializer.serialize_iso(dt) if (dt := item.pop("ignore_data_before", None)) else None
            }

            all_of.append({**item, **{"dynamic_threshold_criterion": props}})
            single_all_of.append({**item, **props})
        else:
            item = cond["static"]
            item["name"] = f"cond{i}"
            props = {
                "operator": item.pop("operator", None),
                "threshold": item.pop("threshold", None)
            }

            all_of.append({**item, **{"static_threshold_criterion": props}})
            single_all_of.append({**item, **props})

    criteria = None
    resource_type, scope_type = _parse_resource_and_scope_type(scopes)
    if scope_type in ['resource_group', 'subscription']:
        if target_resource_type is None or target_resource_region is None:
            raise InvalidArgumentValueError('--target-resource-type and --target-resource-region must be provided.')
        criteria = {"microsoft_azure_monitor_multiple_resource_multiple_metric_criteria": {"all_of": all_of}}
    else:
        if len(scopes) == 1:
            if not is_dynamic_threshold_criterion:
                criteria = {"microsoft_azure_monitor_single_resource_multiple_metric_criteria": {"all_of": single_all_of}}
            else:
                criteria = {"microsoft_azure_monitor_multiple_resource_multiple_metric_criteria": {"all_of": all_of}}
        else:
            criteria = {"microsoft_azure_monitor_multiple_resource_multiple_metric_criteria": {"all_of": all_of}}
            target_resource_type = resource_type
            target_resource_region = target_resource_region if target_resource_region else 'global'

    from ..aaz.latest.monitor.metrics.alert import Create
    return Create(cli_ctx=cmd.cli_ctx)(command_args={
        'resource_group': resource_group_name,
        'name': rule_name,
        'description': description,
        'severity': severity,
        'enabled': not disabled,
        'scopes': scopes,
        'evaluation_frequency': evaluation_frequency,
        'window_size': window_size,
        'criteria': criteria,
        'target_resource_type': target_resource_type,
        'target_resource_region': target_resource_region,
        'actions': actions,
        'tags': tags,
        'location': 'global',
        'auto_mitigate': auto_mitigate
    })


class MetricsAlertUpdate(_MetricsAlertUpdate):
    def __init__(self, loader=None, cli_ctx=None, callbacks=None, **kwargs):
        super().__init__(loader, cli_ctx, callbacks, **kwargs)
        self.add_actions = []
        self.add_conditions = []

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZListArg, AAZStrArg, AAZResourceIdArg, AAZResourceIdArgFormat
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.add_actions = AAZCustomListArg(
            options=["--add-actions"],
            singular_options=["--add-action"],
            arg_group="Action",
            help="Add an action group and optional webhook properties to fire when the alert is triggered.\n\n"
                 "Usage: --add-action ACTION_GROUP_NAME_OR_ID [KEY=VAL [KEY=VAL ...]]\n\n"
                 "Multiple action groups can be specified by using more than one `--add-action` argument."
        )
        args_schema.add_actions.Element = AAZCustomListArg()
        args_schema.add_actions.Element.Element = AAZStrArg()
        args_schema.remove_actions = AAZListArg(
            options=["--remove-actions"],
            arg_group="Action",
            help="Space-separated list of action group names to remove."
        )
        args_schema.remove_actions.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Insights"
                         "/actionGroups/{}"
            )
        )
        args_schema.add_conditions = AAZCustomListArg(
            options=["--add-conditions"],
            singular_options=["--add-condition"],
            arg_group="Condition",
            help="Add a condition which triggers the rule.\n\n"
                 "Usage: --add-condition {avg,min,max,total,count} [NAMESPACE.]METRIC\n"
                 "[{=,!=,>,>=,<,<=} THRESHOLD]\n"
                 "[{>,><,<} dynamic SENSITIVITY VIOLATIONS of EVALUATIONS [since DATETIME]]\n"
                 "[where DIMENSION {includes,excludes} VALUE [or VALUE ...]\n"
                 "[and   DIMENSION {includes,excludes} VALUE [or VALUE ...] ...]]\n\n"
                 "Sensitivity can be 'low', 'medium', 'high'.\n\n"
                 "Violations can be the number of violations to trigger an alert. It should be smaller or equal to evaluation.\n\n"
                 "Evaluations can be the number of evaluation periods for dynamic threshold.\n\n"
                 "Datetime can be the date from which to start learning the metric historical data and calculate the dynamic thresholds (in ISO8601 format).\n\n"
                 "Dimensions can be queried by adding the 'where' keyword and multiple dimensions can be queried by combining them with the 'and' keyword.\n\n"
                 "Values for METRIC, DIMENSION and appropriate THRESHOLD values can be obtained from `az monitor metrics list-definitions` command.\n\n"
                 "Due to server limitation, when an alert rule contains multiple criterias, the use of dimensions is limited to one value per dimension within each criterion.\n\n"
                 "Multiple conditions can be specified by using more than one `--add-condition` argument."
        )
        args_schema.add_conditions.Element = AAZListArg()
        args_schema.add_conditions.Element.Element = AAZStrArg()
        args_schema.remove_conditions = AAZListArg(
            options=["--remove-conditions"],
            arg_group="Condition",
            help="Space-separated list of condition names to remove."
        )
        args_schema.remove_conditions.Element = AAZStrArg()

        return args_schema

    def pre_operations(self):
        def complete_action_group_id(name):
            if is_valid_resource_id(name):
                return name

            return resource_id(
                subscription=self.ctx.subscription_id,
                resource_group=self.ctx.args.resource_group,
                namespace="Microsoft.Insights",
                type="actionGroups",
                name=name
            )

        args = self.ctx.args
        if has_value(args.add_actions):
            self.add_actions = []
            for add_action in args.add_actions:
                values = add_action.to_serialized_data()[0].split()
                action_group_id = complete_action_group_id(values[0])
                try:
                    webhook_property_candidates = dict(x.split('=', 1) for x in values[1:]) if len(values) > 1 else None
                except ValueError:
                    err_msg = "Value of --add-action is invalid. Please refer to --help to get insight of correct format."
                    raise InvalidArgumentValueError(err_msg)

                action = {
                    "action_group_id": action_group_id,
                    "web_hook_properties": webhook_property_candidates
                }
                action["odatatype"] = "Microsoft.WindowsAzure.Management.Monitoring.Alerts.Models." \
                                      "Microsoft.AppInsights.Nexus.DataContracts.Resources.ScheduledQueryRules.Action"

                self.add_actions.append(action)

        if has_value(args.add_conditions):
            err_msg = 'usage error: --condition {avg,min,max,total,count} [NAMESPACE.]METRIC\n' \
                      '                         [{=,!=,>,>=,<,<=} THRESHOLD]\n' \
                      '                         [{<,>,><} dynamic SENSITIVITY VIOLATION of EVALUATION [since DATETIME]]\n' \
                      '                         [where DIMENSION {includes,excludes} VALUE [or VALUE ...]\n' \
                      '                         [and   DIMENSION {includes,excludes} VALUE [or VALUE ...] ...]]\n' \
                      '                         [with skipmetricvalidation]'

            self.add_conditions = []
            for add_condition in args.add_conditions:
                string_val = add_condition.to_serialized_data()[0]
                lexer = MetricAlertConditionLexer(antlr4.InputStream(string_val))
                stream = antlr4.CommonTokenStream(lexer)
                parser = MetricAlertConditionParser(stream)
                tree = parser.expression()

                try:
                    validator = MetricAlertConditionValidator()
                    walker = antlr4.ParseTreeWalker()
                    walker.walk(validator, tree)
                    metric_condition = validator.result()
                    if "static" in metric_condition:
                        # static metric criteria
                        for item in ['time_aggregation', 'metric_name', 'operator', 'threshold']:
                            if item not in metric_condition["static"]:
                                raise InvalidArgumentValueError(err_msg)
                    elif "dynamic" in metric_condition:
                        # dynamic metric criteria
                        for item in ['time_aggregation', 'metric_name', 'operator', 'alert_sensitivity',
                                     'failing_periods']:
                            if item not in metric_condition["dynamic"]:
                                raise InvalidArgumentValueError(err_msg)
                    else:
                        raise NotImplementedError()
                except (AttributeError, TypeError, KeyError):
                    raise InvalidArgumentValueError(err_msg)

                self.add_conditions.append(metric_condition)

    def pre_instance_update(self, instance):
        def get_next_name():
            idx = 0
            while True:
                possible_name = f"cond{idx}"
                match = next((cond for cond in instance.properties.criteria.all_of if cond.name == possible_name), None)
                if match:
                    idx += 1
                    continue

                return possible_name

        args = self.ctx.args
        if has_value(args.remove_actions):
            to_be_removed = set(map(lambda x: x.to_serialized_data().lower(), args.remove_actions))

            new_actions = []
            for action in instance.properties.actions:
                if action.action_group_id.to_serialized_data().lower() not in to_be_removed:
                    new_actions.append(action)

            instance.properties.actions = new_actions

        if has_value(args.add_actions):
            to_be_added = set(map(lambda x: x["action_group_id"].lower(), self.add_actions))

            new_actions = []
            for action in instance.properties.actions:
                if action.action_group_id.to_serialized_data().lower() not in to_be_added:
                    new_actions.append(action)
            new_actions.extend(self.add_actions)

            instance.properties.actions = new_actions

        if has_value(args.remove_conditions):
            to_be_removed = set(map(lambda x: x.to_serialized_data().lower(), args.remove_conditions))

            new_conditions = []
            for cond in instance.properties.criteria.all_of:
                if cond.name.to_serialized_data().lower() not in to_be_removed:
                    new_conditions.append(cond)

            instance.properties.criteria.all_of = new_conditions

        if has_value(args.add_conditions):
            for cond in self.add_conditions:
                if "dynamic" in cond:
                    item = cond["dynamic"]
                    item["name"] = get_next_name()
                    item["criterion_type"] = "DynamicThresholdCriterion"
                    item["ignore_data_before"] = Serializer.serialize_iso(dt) if (dt := item.pop("ignore_data_before", None)) else None

                    instance.properties.criteria.all_of.append(item)
                else:
                    item = cond["static"]
                    item["name"] = get_next_name()
                    item["criterion_type"] = "StaticThresholdCriterion"

                    instance.properties.criteria.all_of.append(item)


def create_metric_alert_dimension(dimension_name, value_list, operator=None):
    values = ' or '.join(value_list)
    return '{} {} {} {}'.format(_metric_alert_dimension_prefix, dimension_name, operator, values)


def create_metric_alert_condition(condition_type, aggregation, metric_name, operator, metric_namespace='',
                                  dimension_list=None, threshold=None, alert_sensitivity=None,
                                  number_of_evaluation_periods=None, min_failing_periods_to_alert=None,
                                  ignore_data_before=None, skip_metric_validation=None):
    if metric_namespace:
        metric_namespace += '.'
    condition = "{} {}'{}' {} ".format(aggregation, metric_namespace, metric_name, operator)
    if condition_type == 'static':
        condition += '{} '.format(threshold)
    elif condition_type == 'dynamic':
        dynamics = 'dynamic {} {} of {} '.format(
            alert_sensitivity, min_failing_periods_to_alert, number_of_evaluation_periods)
        if ignore_data_before:
            dynamics += 'since {} '.format(ignore_data_before)
        condition += dynamics
    else:
        raise NotImplementedError()

    if dimension_list:
        dimensions = ' '.join([t for t in dimension_list if t.strip()])
        if dimensions.startswith(_metric_alert_dimension_prefix):
            dimensions = [t for t in dimensions.split(_metric_alert_dimension_prefix) if t]
            dimensions = 'where' + 'and'.join(dimensions)
        condition += dimensions

    if skip_metric_validation:
        condition += ' with skipmetricvalidation'

    return condition.strip()


def _parse_action_removals(actions):
    """ Separates the combined list of keys to remove into webhooks and emails. """
    flattened = list({x for sublist in actions for x in sublist})
    emails = []
    webhooks = []
    for item in flattened:
        if item.startswith('http://') or item.startswith('https://'):
            webhooks.append(item)
        else:
            emails.append(item)
    return emails, webhooks


def _parse_resource_and_scope_type(scopes):
    from azure.mgmt.core.tools import parse_resource_id

    if not scopes:
        raise InvalidArgumentValueError('scopes cannot be null.')

    namespace = ''
    resource_type = ''
    scope_type = None

    def validate_scope(item_namespace, item_resource_type, item_scope_type):
        if namespace != item_namespace or resource_type != item_resource_type or scope_type != item_scope_type:
            raise InvalidArgumentValueError('Multiple scopes should be the same resource type.')

    def store_scope(item_namespace, item_resource_type, item_scope_type):
        nonlocal namespace
        nonlocal resource_type
        nonlocal scope_type
        namespace = item_namespace
        resource_type = item_resource_type
        scope_type = item_scope_type

    def parse_one_scope_with_action(scope, operation_on_scope):
        result = parse_resource_id(scope)
        if 'namespace' in result and 'resource_type' in result:
            resource_types = [result['type']]
            child_idx = 1
            while 'child_type_{}'.format(child_idx) in result:
                resource_types.append(result['child_type_{}'.format(child_idx)])
                child_idx += 1
            operation_on_scope(result['namespace'], '/'.join(resource_types), 'resource')
        elif 'resource_group' in result:  # It's a resource group.
            operation_on_scope('', '', 'resource_group')
        elif 'subscription' in result:  # It's a subscription.
            operation_on_scope('', '', 'subscription')
        else:
            raise InvalidArgumentValueError('Scope must be a valid resource id.')

    # Store the resource type and scope type from first scope
    parse_one_scope_with_action(scopes[0], operation_on_scope=store_scope)
    # Validate the following scopes
    for item in scopes:
        parse_one_scope_with_action(item, operation_on_scope=validate_scope)

    return namespace + '/' + resource_type, scope_type
